%====================================================================
%
%               Impulse Radio Transmitter + EbNo setting
%
%====================================================================
%Authors    : Ismail Lakkis
%Company    : Wideband Access & I2R
%Date       : July 12, 2005
%Verison    : 1
%====================================================================


clear all;
close all;
format long;

%parameters
%==========
%constants
chanSep     = 494;                      %channel separation in MHz      decimal 494
f0          = 8*chanSep;                %center frequency in MHz        decimal 7,8 or 9 times 494
PRF         = chanSep/32;               %Pulse Repetition Frequency     decimal 15.4375,30.875,61.75,123.5,...
packetLn    = 64*8;                     %packet length in bits          integer 32*8...256*8
dSF         = 7;                        %spreading factor               integer 1...256
dCode       = [-1 0 0 1 0 1 1];         %data spread code               vector 
genPoly     = [1 0 0 1 1;1 1 1 0 1];    %conv. encoder generator poly   matrix

%simulation parameters
EbN0dB      = 3;                        %EbN0 in dB                     decimal 0...30
txOS        = 8;                        %oversampling factor @ tx       integer 4...16
rxOS        = 2;                        %oversampling factor @ tx       integer 4...16
rxDec       = txOS/rxOS;                %receiver decimation factor     integer 1...16
cm          = 0;                        %channel model                  integer 0...9 (0 = awgn)
fs          = chanSep*txOS;             %tx sampling frequency in MHz   decimal

%pulse shaping
pulseTx     = wbaSrrcos(0.25,txOS,8);   %pulse shaping @ Tx             vector
pulseRx     = wbaSrrcos(0.25,rxOS,8);   %pulse shaping @ Rx             vector

%useful parameters
nPulsePerPRF= chanSep/PRF;              %number of pulses per PRF       integer       
nSampsPerPRF= nPulsePerPRF*txOS;        %number of samples per PRF @ Tx integer
nSampsPerBit= nSampsPerPRF*dSF;       %number of samples per bit @ Tx integer

%transmitter
%===========
%tx signal @ PRF level
sync= [];                                       %repeat spreading code SYNCLn times
sfd = [];                                       %start frame delimiter
cir = [];                                       %channel impulse response
dU  = round(rand(1,packetLn/2));                %uncoded data part of the frame
dC  = wbaConvEnc(dU,genPoly);                   %convolutional encode packet
dS  = kron(1-2*dC,dCode(1:dSF));                %spread data @ PRF level
tx  = [sync sfd cir dS];                        %concatenate all parts of frame
%pulse shaping of signal
tx  = kron(tx,[1 zeros(1,nPulsePerPRF-1)]);     %insert zeros between pulses
tx  = kron(tx,[1 zeros(1,txOS-1)]);             %insert zeros between chips (sampling ferequency level)
tx  = conv(tx,pulseTx);                         %transmitted signal after pulse shaping

%wireless uwb channel
%====================
h   = wbaLDRChan(cm,1,chanSep,fs,f0);           %get a (or more) channel realization
rx  = conv(tx,h);L = length(rx);                %received signal after multipath channel
P   = mean(abs(rx.^2));                         %average received power
Ec  = P*nSampsPerBit;                           %energy per single bit in the codeword Ec = P*(DF/PRF)*dSF*txOS
Eb  = 2*Ec;                                     %energy per information bit (Proakis P.440)
N0  = Eb*(10^(-EbN0dB/10));                     %noise variance
w   = sqrt(N0/2)*(randn(1,L) + j*randn(1,L));   %AWGN with variance N0/2
rx  = rx + w;                                   %received signal
clear w;

%receiver (verify BER for AWGN)
%==============================
if (cm == 0),
    %receiver matched filter
    rx      = conv(rx,pulseTx);
    %decimate down to one sample per PRF assuming perfect knowldege of timing
    rx      = rx([0:packetLn*dSF-1]*nSampsPerPRF + length(pulseTx));
    rx      = reshape(rx,dSF,packetLn);         %each column corresponds to one spread symbol
    dCe     = real(dCode*rx);                   %despreading + hard decision
    %dUe= nwConvDec(dCe);                       %convolutional decode packet goes here
    dCe     = 0.5 - 0.5*sign(dCe);              %mapping from +1,-1 to 0,1
    BERSim  = sum(abs(dC-dCe))/packetLn;        %simulated BER
    EcN0dB  = EbN0dB - 3;                       %for the coded bit in dB
    EcN0    = 10^(EcN0dB/10);                   %for the coded bit in linear
    BERThe  = 0.5*erfc(sqrt(EcN0));             %theoretical ber
    fprintf('EcN0dB = %f, AWGN BER-Sim = %f, AWGN BER-The = %f\n',EbN0dB-3,BERSim,BERThe);
end;

